"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var factory_1 = require("../factory");
var ControllerInvocation = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerInvocation, _super);
    function ControllerInvocation(controllerInfo) {
        var _this = _super.call(this) || this;
        _this.controllerInfo = controllerInfo;
        return _this;
    }
    ControllerInvocation.prototype.proceed = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var controller = factory_1.ControllerFactory.resolve(_this.controllerInfo, _this.facade.dependencyResolver);
            if (_this.context.contextType == "HttpRequest")
                controller.request = _this.context;
            else
                controller.handshake = _this.context;
            controller.invocation = _this;
            var method = controller[_this.controllerInfo.methodMetaData.name];
            if (_this.facade.autoValidation && _this.controllerInfo.methodMetaData.parameters.length > 0 && !controller.validator.isValid())
                resolve(new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json"));
            else {
                var result = method.apply(controller, _this.parameters);
                Promise.resolve(result)
                    .then(function (x) {
                    if (x instanceof Core.ActionResult)
                        return x;
                    if (_this.controllerInfo.classMetaData.baseClass == "ApiController") {
                        return new Core.ActionResult(x, undefined, "application/json");
                    }
                    return new Core.ActionResult(x);
                })
                    .then(resolve)
                    .catch(reject);
            }
        });
    };
    return ControllerInvocation;
}(Core.Invocation));
exports.ControllerInvocation = ControllerInvocation;
